#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Bar Color based on modified Heiken Ashi technique.
    /// </summary>
    [Description("Bar Color based on modified Heiken Ashi technique.")]
    public class HAcolor : Indicator
    {
        #region Variables
        // Wizard generated variables
        // User defined variables (add any user defined variables below)
		private Color	barColorUp 		= Color.White;
		private Color	barColorDown 	= Color.Red;
		double close, close1, open, open1;
		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar == 0)
			{
				close = close1 = 0;
				open = open1 = 0;
				return;
			}
			
			close = (Open[0] + High[0] + Low[0] + Close[0]) / 4.0;
			open = (close1 + open1) / 2.0;
			close1 = close;
			open1 = open;
			
			BarColor = (close > open ? BarColorUp : BarColorDown);			
        }

        #region Properties
		[XmlIgnore()]
		[Description("Color of up bars.")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Up color")]
		public Color BarColorUp
		{
			get { return barColorUp; }
			set { barColorUp = value; }
		}

		[XmlIgnore()]
		[Description("Color of down bars.")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Down color")]
		public Color BarColorDown
		{
			get { return barColorDown; }
			set { barColorDown = value; }
		}

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private HAcolor[] cacheHAcolor = null;

        private static HAcolor checkHAcolor = new HAcolor();

        /// <summary>
        /// Bar Color based on modified Heiken Ashi technique.
        /// </summary>
        /// <returns></returns>
        public HAcolor HAcolor()
        {
            return HAcolor(Input);
        }

        /// <summary>
        /// Bar Color based on modified Heiken Ashi technique.
        /// </summary>
        /// <returns></returns>
        public HAcolor HAcolor(Data.IDataSeries input)
        {

            if (cacheHAcolor != null)
                for (int idx = 0; idx < cacheHAcolor.Length; idx++)
                    if (cacheHAcolor[idx].EqualsInput(input))
                        return cacheHAcolor[idx];

            HAcolor indicator = new HAcolor();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.SetUp();

            HAcolor[] tmp = new HAcolor[cacheHAcolor == null ? 1 : cacheHAcolor.Length + 1];
            if (cacheHAcolor != null)
                cacheHAcolor.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheHAcolor = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Bar Color based on modified Heiken Ashi technique.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.HAcolor HAcolor()
        {
            return _indicator.HAcolor(Input);
        }

        /// <summary>
        /// Bar Color based on modified Heiken Ashi technique.
        /// </summary>
        /// <returns></returns>
        public Indicator.HAcolor HAcolor(Data.IDataSeries input)
        {
            return _indicator.HAcolor(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Bar Color based on modified Heiken Ashi technique.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.HAcolor HAcolor()
        {
            return _indicator.HAcolor(Input);
        }

        /// <summary>
        /// Bar Color based on modified Heiken Ashi technique.
        /// </summary>
        /// <returns></returns>
        public Indicator.HAcolor HAcolor(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.HAcolor(input);
        }

    }
}
#endregion
