#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// MACD Histogram
    /// </summary>
    [Description("MACD Histogram")]
    public class MacdHist : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int fast = 12; // Default setting for Fast
            private int slow = 26; // Default setting for Slow
            private int smooth = 9; // Default setting for Smooth

		private	DataSeries		fastEma;
		private	DataSeries		slowEma;
		private	DataSeries		signal;
		
		private double macd, macdH;  // calculated values

		#endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.White),6) , PlotStyle.Bar, "AboveZero"));
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.Red), 6), PlotStyle.Bar, "BelowZero"));
            Add(new Line(Color.FromKnownColor(KnownColor.DarkGray), 0, "ZeroLine"));
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;
			
			// Set the threshold values for each plot
			Plots[0].Min = 0;
			Plots[1].Max = 0;
			
			fastEma	= new DataSeries(this);
			slowEma	= new DataSeries(this);
			signal	= new DataSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar == 0)
			{
				fastEma.Set(Input[0]);
				slowEma.Set(Input[0]);
				signal.Set(Input[0]);
				AboveZero.Set(0);
				BelowZero.Set(0);
			}
			else
			{
				fastEma.Set((2.0 / (1 + Fast)) * Input[0] + (1 - (2.0 / (1 + Fast))) * fastEma[1]);
				slowEma.Set((2.0 / (1 + Slow)) * Input[0] + (1 - (2.0 / (1 + Slow))) * slowEma[1]);

				macd = fastEma[0] - slowEma[0];
				signal.Set((2.0 / (1 + Smooth)) * macd + (1 - (2.0 / (1 + Smooth))) * signal[1]);
								
				macdH = macd - signal[0];
				
				// plot the histogram bar
				AboveZero.Set(macdH);
				BelowZero.Set(macdH);
			}
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries AboveZero
        {
            get { return Values[0]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries BelowZero
        {
            get { return Values[1]; }
        }

        [Description("Fast EMA period")]
        [Category("Parameters")]
        public int Fast
        {
            get { return fast; }
            set { fast = Math.Max(1, value); }
        }

        [Description("Slow EMA period")]
        [Category("Parameters")]
        public int Slow
        {
            get { return slow; }
            set { slow = Math.Max(1, value); }
        }

        [Description("Smoothing")]
        [Category("Parameters")]
        public int Smooth
        {
            get { return smooth; }
            set { smooth = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private MacdHist[] cacheMacdHist = null;

        private static MacdHist checkMacdHist = new MacdHist();

        /// <summary>
        /// MACD Histogram
        /// </summary>
        /// <returns></returns>
        public MacdHist MacdHist(int fast, int slow, int smooth)
        {
            return MacdHist(Input, fast, slow, smooth);
        }

        /// <summary>
        /// MACD Histogram
        /// </summary>
        /// <returns></returns>
        public MacdHist MacdHist(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            checkMacdHist.Fast = fast;
            fast = checkMacdHist.Fast;
            checkMacdHist.Slow = slow;
            slow = checkMacdHist.Slow;
            checkMacdHist.Smooth = smooth;
            smooth = checkMacdHist.Smooth;

            if (cacheMacdHist != null)
                for (int idx = 0; idx < cacheMacdHist.Length; idx++)
                    if (cacheMacdHist[idx].Fast == fast && cacheMacdHist[idx].Slow == slow && cacheMacdHist[idx].Smooth == smooth && cacheMacdHist[idx].EqualsInput(input))
                        return cacheMacdHist[idx];

            MacdHist indicator = new MacdHist();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Fast = fast;
            indicator.Slow = slow;
            indicator.Smooth = smooth;
            indicator.SetUp();

            MacdHist[] tmp = new MacdHist[cacheMacdHist == null ? 1 : cacheMacdHist.Length + 1];
            if (cacheMacdHist != null)
                cacheMacdHist.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheMacdHist = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// MACD Histogram
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MacdHist MacdHist(int fast, int slow, int smooth)
        {
            return _indicator.MacdHist(Input, fast, slow, smooth);
        }

        /// <summary>
        /// MACD Histogram
        /// </summary>
        /// <returns></returns>
        public Indicator.MacdHist MacdHist(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            return _indicator.MacdHist(input, fast, slow, smooth);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// MACD Histogram
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MacdHist MacdHist(int fast, int slow, int smooth)
        {
            return _indicator.MacdHist(Input, fast, slow, smooth);
        }

        /// <summary>
        /// MACD Histogram
        /// </summary>
        /// <returns></returns>
        public Indicator.MacdHist MacdHist(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.MacdHist(input, fast, slow, smooth);
        }

    }
}
#endregion
